from scpython.model.article import Article
from scpython.options import Branch, Language
import re


class ScpArticle(Article):
    """Represents an SCP Article on Wikidot.

    Attributes:
        page_source (str): The SCP's text.
        number (int): The SCP's number.
        branch (scpython.options.Branch): The SCP's original branch.
        language (scpython.options.Language): The fetched SCP's language.
        is_joke (boolean): Whether the SCP has a -J flag.
        full_code (str): The SCP's full code (SCP-XXXX-LANG-J).
    """

    def __init__(self, data):
        """Parses the raw values from the data.

        Args:
            data (dict): The raw data from Wikidot.
        """
        super().__init__(data)

        self.page_source = data["page_source"]
        self.is_joke = "joke" in self.tags
        self.full_code = data["full_code"]
        self.number = int(re.search(r"(\d+)", self.full_code).group(1))
        self.language = data["language"]
        self.branch = data["branch"]

    def toJson(self):
        article = super().toJson()
        return {
            **article,
            "page_source": self.page_source,
            "is_joke": self.is_joke,
            "number": self.number,
            "full_code": self.full_code,
            "language": self.language,
            "branch": self.branch
        }



import scpython.model.scparticle
from scpython.options import Branch, Language
from scpython.util.enum import getScpBranch, getItemByName

from threading import Thread
import json
import html
import urllib3
import re


class Client:
    """
    A client to communicate with Wikidot.

    Attributes:
        __token (str): The user's identity.
        __pool: The pool to make requests with.
    """

    __AJAX_CONNECTOR = "/ajax-module-connector.php"

    def __init__(self, wikidot_token7):
        """Registers the token.

        Args:
            wikidot_token7 (str): The user's Wikidot token.
        """
        self.__token = wikidot_token7
        self.__pool = urllib3.PoolManager()

    def getScpArticle(self, code, branch=None, language=None):
        """Gets an SCP from a certain wiki.

        Args:
            code (str): The SCP's code.
            branch (scpython.options.Branch): The SCP's original branch.
            language (scpython.options.Language): The language to return the SCP in.
        """
        branch = branch if branch is not None and isinstance(branch, Branch) \
            else getScpBranch(code)

        if language is None:
            language = getItemByName(Language, branch.name)
            if branch is Branch.INT:
                language = Language.EN

        wiki_url = branch.value
        url = wiki_url + code

        response = self.__pool.request(
            "GET",
            url,
            headers={"Cookie": f"wikidot_token7={self.__token}"}
        )

        if response.status == 200:
            raw_html = response.data.decode("utf-8")

            regex = {
                "id": r"WIKIREQUEST.info.pageId = (\d+);",
                "page_version": r"<div id=\"page-info\">\D+?: (\d+?), \D+?: .+?<\/div>",
                "last_change": r"<div id=\"page-info\">.+?<span class=\".+?\">(.+?)<\/span><\/div>",
                "rating": r"<span class=\"rate-points\">.+?<span class=\".+?\">((?:\+|-)\d+?)<\/span><\/span>",
            }

            scp_data = {
                "url": url,
            }
            for var in regex:
                result = re.search(regex[var], raw_html).group(1)
                scp_data[var] = result

            raw_tags = re.search(r"<div class=\"page-tags\">\s+<span>\s+(?:<a href=\".+?\">.+?</a>)+", raw_html) \
                .group(0)
            tags = re.findall(r"<a href=\".+?\">(.+?)</a>", raw_tags)
            scp_data = {
                **scp_data,
                "tags": tags
            }

            req_poster = Client.__RequestThread(self.__pool)
            req_poster.set_request(
                "POST",
                wiki_url + Client.__AJAX_CONNECTOR,
                headers={"Cookie": f"wikidot_token7={self.__token}"},
                fields={
                    "page_id": scp_data["id"],
                    "moduleName": "history/PageRevisionListModule",
                    "wikidot_token7": self.__token,
                    "options": "{\"all\":true}",
                    "page": int(scp_data["page_version"])+1,
                    "perpage": "1",
                }
            )
            req_poster.start()

            req_page_source = Client.__RequestThread(self.__pool)
            req_page_source.set_request(
                "POST",
                wiki_url + Client.__AJAX_CONNECTOR,
                headers={"Cookie": f"wikidot_token7={self.__token}"},
                fields={
                    "page_id": scp_data["id"],
                    "moduleName": "viewsource/ViewSourceModule",
                    "wikidot_token7": self.__token,
                }
            )
            req_page_source.start()

            req_poster.join()
            req_page_source.join()

            res_poster = req_poster.getResponse()
            author = None if not res_poster \
                else re.search(
                    r"<a href=\".+?\" onclick=\".+?\" ?>(.+)</a></span></td>",
                    json.loads(res_poster.decode("utf-8"))["body"]
                ).group(1)

            res_page_source = req_page_source.getResponse()
            page_source = "" if not res_page_source \
                else json.loads(res_page_source.decode("utf-8"))["body"]
            page_source = html.unescape(
                re.match(r"<h1>.+?</h1>\s+<div class=\"page-source\">([\w\W]+)</div>", page_source)
                .group(1)
                .replace("<br />", "")
            )

            scp_data = {
                **scp_data,
                "branch": branch,
                "full_code": code,
                "page_source": page_source,
                "poster": author,
                "language": language
            }

            return scpython.model.scparticle.ScpArticle(scp_data)
        else:
            raise Exception("Invalid response status")

    class __RequestThread(Thread):
        """A Thread to make requests, to speed up things.

        Attributes:
            __pool: The pool to make requests with.
            __method (str): The HTTP Method.
            __url (str): The URL to send the request to.
            __request_kwargs (dict): The optional parameters to send to the request.
            __response: The request response.
        """

        def __init__(self, pool):
            super().__init__()

            self.__pool = pool
            self.__method = None
            self.__url = None
            self.__request_kwargs = {}

            self.__response = None

        def set_request(self, method, url, **kwargs):
            self.__method = method
            self.__url = url
            self.__request_kwargs = kwargs if kwargs else {}

        def run(self):
            if self.__method and self.__url:
                response = self.__pool.request(self.__method, self.__url, **self.__request_kwargs)
                if response.status == 200:
                    self.__response = response.data
                    return

            self.__response = None

        def getResponse(self):
            return self.__response



from abc import ABC
from datetime import datetime


class Article(ABC):
    """Represents any Article on Wikidot.

    Attributes:
        id (int): The article ID.
        rating (int): The article's rating.
        url (str): The article's URL.
        tags (str[]): The article's tags.
        poster (str): The user who first posted the article.
        page_version (int): The number of changes the page went through.
        last_change (date): The day the page was last changed.
    """

    def __init__(self, data):
        """Parses the raw values from the data.

        Args:
            data (dict): The raw data from Wikidot.
        """
        self.id = int(data["id"])
        self.url = data["url"]
        self.tags = data["tags"]
        self.page_version = int(data["page_version"])
        self.last_change = data["last_change"] if isinstance(data["last_change"], datetime) \
            else datetime.strptime(data["last_change"], "%d %b %Y %H:%M")
        self.poster = data["poster"]

        self.rating = None
        rating = data["rating"]
        if isinstance(rating, int):
            self.rating = rating
        elif isinstance(rating, str):
            if rating.isnumeric():
                self.rating = rating
            elif len(rating) > 2 and rating[0] == "+":
                self.rating = int(rating[1:])

    def toJson(self):
        return {
            "id": self.id,
            "rating": self.rating,
            "url": self.url,
            "tags": self.tags,
            "poster": self.poster,
            "page_version": self.page_version,
            "last_change": self.last_change,
        }




from enum import Enum


class Branch(Enum):
    INT = "http://scp-int.wikidot.com/"
    EN = "http://www.scp-wiki.net/"
    RU = "http://scp-ru.wikidot.com/"
    KO = "http://ko.scp-wiki.net/"
    CN = "http://scp-wiki-cn.wikidot.com/"
    FR = "http://fondationscp.wikidot.com/"
    PL = "http://scp-wiki.net.pl/"
    ES = "http://lafundacionscp.wikidot.com/"
    TH = "http://scp-th.wikidot.com/"
    JP = "http://scp-jp.wikidot.com/"
    DE = "http://scp-wiki-de.wikidot.com/"
    IT = "http://fondazionescp.wikidot.com/"
    UA = "http://scp-ukrainian.wikidot.com/"
    PT = "http://scp-pt-br.wikidot.com/"
    BR = PT
    CZ = "http://scp-cs.wikidot.com/"



from enum import Enum


class Language(Enum):
    EN = "EN"
    RU = "RU"
    KO = "KO"
    CN = "CN"
    FR = "FR"
    PL = "PL"
    ES = "ES"
    TH = "TH"
    JP = "JP"
    DE = "DE"
    IT = "IT"
    UA = "UA"
    PT = "PT"
    BR = PT
    CZ = "CZ"



from scpython.options import Branch, Language
import re


def getScpBranch(code):
    """Detects an SCP's branch by its code.

    Args:
        code (str): The SCP's code.

    Return:
        (scpython.options.Branch): The SCP's branch.
    """
    languages = [l.name for l in list(Branch)]
    regex = f"-({'|'.join(languages)})"

    ret = Branch.EN
    match = re.search(regex, code)
    if match:
        b = match.group(1)
        ret = getItemByName(Branch, b)
    return ret


def getItemByName(enum, name):
    """Gets an enum item by its variable name.

    Args:
        enum (Enum): The enum to check.
        name (str): The name of the variable to find.

    Return:
        The enum variable, or None.
    """
    for x in list(enum):
        if x.name == name:
            return x
    return None
